# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....organization_providers.resources.v_2.types.organization_provider_id import OrganizationProviderId
from .raw_client import AsyncRawV2Client, RawV2Client
from .types.compute_all_in_network_providers_request import ComputeAllInNetworkProvidersRequest
from .types.compute_all_in_network_providers_response import ComputeAllInNetworkProvidersResponse
from .types.expected_network_status_request_v_2 import ExpectedNetworkStatusRequestV2
from .types.expected_network_status_response_v_2 import ExpectedNetworkStatusResponseV2

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V2Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV2Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV2Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV2Client
        """
        return self._raw_client

    def compute_for_rendering_provider(
        self,
        rendering_provider_id: OrganizationProviderId,
        *,
        request: ExpectedNetworkStatusRequestV2,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExpectedNetworkStatusResponseV2:
        """
        Computes the expected network status for a given rendering provider.
        This endpoint is not available to all customers. Reach out to the Candid sales team
        to discuss enabling this endpoint if it is not available for your organization.

        Parameters
        ----------
        rendering_provider_id : OrganizationProviderId

        request : ExpectedNetworkStatusRequestV2

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExpectedNetworkStatusResponseV2

        Examples
        --------
        import datetime
        import uuid

        from candid import CandidApiClient
        from candid.resources.commons import (
            FacilityTypeCode,
            InsuranceTypeCode,
            State,
            StreetAddressShortZip,
        )
        from candid.resources.expected_network_status.resources.v_2 import (
            ExpectedNetworkStatusRequestV2,
            ExpectedNetworkStatusSubscriberInformation,
            InsuranceType,
            InsuranceTypeCodes_InsuranceTypeCode,
            LineOfBusiness,
            ServiceType,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.expected_network_status.v_2.compute_for_rendering_provider(
            rendering_provider_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=ExpectedNetworkStatusRequestV2(
                service_type=ServiceType.NEW_PATIENT_VIDEO_APPT,
                place_of_service_code=FacilityTypeCode.PHARMACY,
                subscriber_information=ExpectedNetworkStatusSubscriberInformation(
                    payer_uuid=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    member_id="member_id",
                    insurance_type=InsuranceType(
                        line_of_business=LineOfBusiness.MEDICARE,
                        insurance_type_codes=InsuranceTypeCodes_InsuranceTypeCode(
                            value=InsuranceTypeCode.C_01
                        ),
                    ),
                ),
                patient_address=StreetAddressShortZip(
                    address_1="address1",
                    city="city",
                    state=State.AA,
                    zip_code="zip_code",
                ),
                billing_provider_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                organization_service_facility_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                date_of_service=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
            ),
        )
        """
        _response = self._raw_client.compute_for_rendering_provider(
            rendering_provider_id, request=request, request_options=request_options
        )
        return _response.data

    def compute_all_in_network_providers(
        self, *, request: ComputeAllInNetworkProvidersRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> ComputeAllInNetworkProvidersResponse:
        """
        Computes all the in network providers for a given set of inputs.
        This endpoint is not available to all customers. Reach out to the Candid sales team
        to discuss enabling this endpoint if it is not available for your organization.

        Parameters
        ----------
        request : ComputeAllInNetworkProvidersRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ComputeAllInNetworkProvidersResponse

        Examples
        --------
        import datetime
        import uuid

        from candid import CandidApiClient
        from candid.resources.commons import (
            FacilityTypeCode,
            InsuranceTypeCode,
            State,
            StreetAddressShortZip,
        )
        from candid.resources.expected_network_status.resources.v_2 import (
            ComputeAllInNetworkProvidersRequest,
            ExpectedNetworkStatusSubscriberInformation,
            InsuranceType,
            InsuranceTypeCodes_InsuranceTypeCode,
            LineOfBusiness,
            ServiceType,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.expected_network_status.v_2.compute_all_in_network_providers(
            request=ComputeAllInNetworkProvidersRequest(
                service_type=ServiceType.NEW_PATIENT_VIDEO_APPT,
                place_of_service_code=FacilityTypeCode.PHARMACY,
                subscriber_information=ExpectedNetworkStatusSubscriberInformation(
                    payer_uuid=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    member_id="member_id",
                    insurance_type=InsuranceType(
                        line_of_business=LineOfBusiness.MEDICARE,
                        insurance_type_codes=InsuranceTypeCodes_InsuranceTypeCode(
                            value=InsuranceTypeCode.C_01
                        ),
                    ),
                ),
                patient_address=StreetAddressShortZip(
                    address_1="address1",
                    city="city",
                    state=State.AA,
                    zip_code="zip_code",
                ),
                billing_provider_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                organization_service_facility_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                date_of_service=datetime.date.fromisoformat(
                    "2023-01-15",
                ),
            ),
        )
        """
        _response = self._raw_client.compute_all_in_network_providers(request=request, request_options=request_options)
        return _response.data


class AsyncV2Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV2Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV2Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV2Client
        """
        return self._raw_client

    async def compute_for_rendering_provider(
        self,
        rendering_provider_id: OrganizationProviderId,
        *,
        request: ExpectedNetworkStatusRequestV2,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ExpectedNetworkStatusResponseV2:
        """
        Computes the expected network status for a given rendering provider.
        This endpoint is not available to all customers. Reach out to the Candid sales team
        to discuss enabling this endpoint if it is not available for your organization.

        Parameters
        ----------
        rendering_provider_id : OrganizationProviderId

        request : ExpectedNetworkStatusRequestV2

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ExpectedNetworkStatusResponseV2

        Examples
        --------
        import asyncio
        import datetime
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.commons import (
            FacilityTypeCode,
            InsuranceTypeCode,
            State,
            StreetAddressShortZip,
        )
        from candid.resources.expected_network_status.resources.v_2 import (
            ExpectedNetworkStatusRequestV2,
            ExpectedNetworkStatusSubscriberInformation,
            InsuranceType,
            InsuranceTypeCodes_InsuranceTypeCode,
            LineOfBusiness,
            ServiceType,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.expected_network_status.v_2.compute_for_rendering_provider(
                rendering_provider_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=ExpectedNetworkStatusRequestV2(
                    service_type=ServiceType.NEW_PATIENT_VIDEO_APPT,
                    place_of_service_code=FacilityTypeCode.PHARMACY,
                    subscriber_information=ExpectedNetworkStatusSubscriberInformation(
                        payer_uuid=uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                        member_id="member_id",
                        insurance_type=InsuranceType(
                            line_of_business=LineOfBusiness.MEDICARE,
                            insurance_type_codes=InsuranceTypeCodes_InsuranceTypeCode(
                                value=InsuranceTypeCode.C_01
                            ),
                        ),
                    ),
                    patient_address=StreetAddressShortZip(
                        address_1="address1",
                        city="city",
                        state=State.AA,
                        zip_code="zip_code",
                    ),
                    billing_provider_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    organization_service_facility_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    date_of_service=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.compute_for_rendering_provider(
            rendering_provider_id, request=request, request_options=request_options
        )
        return _response.data

    async def compute_all_in_network_providers(
        self, *, request: ComputeAllInNetworkProvidersRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> ComputeAllInNetworkProvidersResponse:
        """
        Computes all the in network providers for a given set of inputs.
        This endpoint is not available to all customers. Reach out to the Candid sales team
        to discuss enabling this endpoint if it is not available for your organization.

        Parameters
        ----------
        request : ComputeAllInNetworkProvidersRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ComputeAllInNetworkProvidersResponse

        Examples
        --------
        import asyncio
        import datetime
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.commons import (
            FacilityTypeCode,
            InsuranceTypeCode,
            State,
            StreetAddressShortZip,
        )
        from candid.resources.expected_network_status.resources.v_2 import (
            ComputeAllInNetworkProvidersRequest,
            ExpectedNetworkStatusSubscriberInformation,
            InsuranceType,
            InsuranceTypeCodes_InsuranceTypeCode,
            LineOfBusiness,
            ServiceType,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.expected_network_status.v_2.compute_all_in_network_providers(
                request=ComputeAllInNetworkProvidersRequest(
                    service_type=ServiceType.NEW_PATIENT_VIDEO_APPT,
                    place_of_service_code=FacilityTypeCode.PHARMACY,
                    subscriber_information=ExpectedNetworkStatusSubscriberInformation(
                        payer_uuid=uuid.UUID(
                            "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                        ),
                        member_id="member_id",
                        insurance_type=InsuranceType(
                            line_of_business=LineOfBusiness.MEDICARE,
                            insurance_type_codes=InsuranceTypeCodes_InsuranceTypeCode(
                                value=InsuranceTypeCode.C_01
                            ),
                        ),
                    ),
                    patient_address=StreetAddressShortZip(
                        address_1="address1",
                        city="city",
                        state=State.AA,
                        zip_code="zip_code",
                    ),
                    billing_provider_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    organization_service_facility_id=uuid.UUID(
                        "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                    ),
                    date_of_service=datetime.date.fromisoformat(
                        "2023-01-15",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.compute_all_in_network_providers(
            request=request, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class TaskType(str, enum.Enum):
    CUSTOMER_DATA_REQUEST = "CUSTOMER_DATA_REQUEST"
    CLAIM_SUBMISSION_RUN_VALIDATIONS_FAILURE = "CLAIM_SUBMISSION_RUN_VALIDATIONS_FAILURE"
    CLAIM_SUBMISSION_SUBMIT_CLAIM_FAILURE = "CLAIM_SUBMISSION_SUBMIT_CLAIM_FAILURE"
    CLAIM_SUBMISSION_TRANSFORM_FAILURE = "CLAIM_SUBMISSION_TRANSFORM_FAILURE"
    CODE_CLAIM = "CODE_CLAIM"
    CODE_CLAIM_REVIEW = "CODE_CLAIM_REVIEW"
    ELIGIBILITY = "ELIGIBILITY"
    CLAIM_FOLLOW_UP = "CLAIM_FOLLOW_UP"
    REJECTION_RESOLUTION = "REJECTION_RESOLUTION"
    PAYER_CONFIGURATION_ERROR = "PAYER_CONFIGURATION_ERROR"
    DENIAL_RESOLUTION = "DENIAL_RESOLUTION"
    MISSING_ENROLLMENT = "MISSING_ENROLLMENT"
    _UNKNOWN = "__TASKTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "TaskType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        customer_data_request: typing.Callable[[], T_Result],
        claim_submission_run_validations_failure: typing.Callable[[], T_Result],
        claim_submission_submit_claim_failure: typing.Callable[[], T_Result],
        claim_submission_transform_failure: typing.Callable[[], T_Result],
        code_claim: typing.Callable[[], T_Result],
        code_claim_review: typing.Callable[[], T_Result],
        eligibility: typing.Callable[[], T_Result],
        claim_follow_up: typing.Callable[[], T_Result],
        rejection_resolution: typing.Callable[[], T_Result],
        payer_configuration_error: typing.Callable[[], T_Result],
        denial_resolution: typing.Callable[[], T_Result],
        missing_enrollment: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is TaskType.CUSTOMER_DATA_REQUEST:
            return customer_data_request()
        if self is TaskType.CLAIM_SUBMISSION_RUN_VALIDATIONS_FAILURE:
            return claim_submission_run_validations_failure()
        if self is TaskType.CLAIM_SUBMISSION_SUBMIT_CLAIM_FAILURE:
            return claim_submission_submit_claim_failure()
        if self is TaskType.CLAIM_SUBMISSION_TRANSFORM_FAILURE:
            return claim_submission_transform_failure()
        if self is TaskType.CODE_CLAIM:
            return code_claim()
        if self is TaskType.CODE_CLAIM_REVIEW:
            return code_claim_review()
        if self is TaskType.ELIGIBILITY:
            return eligibility()
        if self is TaskType.CLAIM_FOLLOW_UP:
            return claim_follow_up()
        if self is TaskType.REJECTION_RESOLUTION:
            return rejection_resolution()
        if self is TaskType.PAYER_CONFIGURATION_ERROR:
            return payer_configuration_error()
        if self is TaskType.DENIAL_RESOLUTION:
            return denial_resolution()
        if self is TaskType.MISSING_ENROLLMENT:
            return missing_enrollment()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClaimStatusCodeCreate(str, enum.Enum):
    """
    See https://www.stedi.com/edi/x12/element/1029
    """

    PROCESSED_AS_PRIMARY = "1"
    PROCESSED_AS_SECONDARY = "2"
    PROCESSED_AS_TERTIARY = "3"
    DENIED = "4"
    PROCESSED_AS_PRIMARY_FORWARDED = "19"
    PROCESSED_AS_SECONDARY_FORWARDED = "20"
    PROCESSED_AS_TERTIARY_FORWARDED = "21"
    REVERSAL_OF_PREVIOUS_PAYMENT = "22"
    NOT_OUR_CLAIM_FORWARDED = "23"
    _UNKNOWN = "__CLAIMSTATUSCODECREATE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ClaimStatusCodeCreate":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        processed_as_primary: typing.Callable[[], T_Result],
        processed_as_secondary: typing.Callable[[], T_Result],
        processed_as_tertiary: typing.Callable[[], T_Result],
        denied: typing.Callable[[], T_Result],
        processed_as_primary_forwarded: typing.Callable[[], T_Result],
        processed_as_secondary_forwarded: typing.Callable[[], T_Result],
        processed_as_tertiary_forwarded: typing.Callable[[], T_Result],
        reversal_of_previous_payment: typing.Callable[[], T_Result],
        not_our_claim_forwarded: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ClaimStatusCodeCreate.PROCESSED_AS_PRIMARY:
            return processed_as_primary()
        if self is ClaimStatusCodeCreate.PROCESSED_AS_SECONDARY:
            return processed_as_secondary()
        if self is ClaimStatusCodeCreate.PROCESSED_AS_TERTIARY:
            return processed_as_tertiary()
        if self is ClaimStatusCodeCreate.DENIED:
            return denied()
        if self is ClaimStatusCodeCreate.PROCESSED_AS_PRIMARY_FORWARDED:
            return processed_as_primary_forwarded()
        if self is ClaimStatusCodeCreate.PROCESSED_AS_SECONDARY_FORWARDED:
            return processed_as_secondary_forwarded()
        if self is ClaimStatusCodeCreate.PROCESSED_AS_TERTIARY_FORWARDED:
            return processed_as_tertiary_forwarded()
        if self is ClaimStatusCodeCreate.REVERSAL_OF_PREVIOUS_PAYMENT:
            return reversal_of_previous_payment()
        if self is ClaimStatusCodeCreate.NOT_OUR_CLAIM_FORWARDED:
            return not_our_claim_forwarded()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.invoice_id import InvoiceId
from .....financials.types.allocation import Allocation
from .....financials.types.refund_reason import RefundReason
from .....non_insurance_payers.resources.v_1.types.non_insurance_payer import NonInsurancePayer
from .non_insurance_payer_refund_id import NonInsurancePayerRefundId


class NonInsurancePayerRefund(UniversalBaseModel):
    non_insurance_payer_refund_id: NonInsurancePayerRefundId
    non_insurance_payer: NonInsurancePayer
    amount_cents: int
    refund_timestamp: typing.Optional[dt.datetime] = None
    refund_note: typing.Optional[str] = None
    check_number: typing.Optional[str] = None
    allocations: typing.List[Allocation]
    refund_reason: typing.Optional[RefundReason] = None
    invoice_id: typing.Optional[InvoiceId] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.primitive import Primitive
from .....commons.types.schema_id import SchemaId


class SchemaValidationError_SchemaNameAlreadyExists(UniversalBaseModel):
    type: typing.Literal["schema_name_already_exists"] = "schema_name_already_exists"
    name: str
    id: SchemaId

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class SchemaValidationError_KeyNameAlreadyExists(UniversalBaseModel):
    type: typing.Literal["key_name_already_exists"] = "key_name_already_exists"
    key: str
    value_type: Primitive

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


SchemaValidationError = typing.Union[
    SchemaValidationError_SchemaNameAlreadyExists, SchemaValidationError_KeyNameAlreadyExists
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import GetExportsResponse
from .errors import (
    ExportDateTooEarlyError,
    ExportFilesUnavailableError,
    ExportNotYetAvailableError,
    MissingDailyIncrementalExportFileError,
)

__all__ = [
    "ExportDateTooEarlyError",
    "ExportFilesUnavailableError",
    "ExportNotYetAvailableError",
    "GetExportsResponse",
    "MissingDailyIncrementalExportFileError",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from .diagnosis_id import DiagnosisId
from .standalone_diagnosis_create import StandaloneDiagnosisCreate


class Diagnosis(StandaloneDiagnosisCreate):
    """
    Examples
    --------
    import datetime
    import uuid

    from candid.resources.diagnoses import Diagnosis, DiagnosisTypeCode

    Diagnosis(
        diagnosis_id=uuid.UUID(
            "5c770e00-4bbf-42af-a73f-99c5e91fc0db",
        ),
        created_at=datetime.datetime.fromisoformat(
            "2023-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2023-01-01 00:00:00+00:00",
        ),
        encounter_id=uuid.UUID(
            "3f63985b-51a4-4dd4-9418-7d50b2520792",
        ),
        name="John Doe",
        code_type=DiagnosisTypeCode.ABF,
        code="I10",
    )
    """

    diagnosis_id: DiagnosisId
    created_at: dt.datetime
    updated_at: dt.datetime

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .carc import Carc
from .claim_adjustment_reason_code import ClaimAdjustmentReasonCode
from .rarc import Rarc
from .remittance_advice_remark_code import RemittanceAdviceRemarkCode

__all__ = ["Carc", "ClaimAdjustmentReasonCode", "Rarc", "RemittanceAdviceRemarkCode"]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations


class CandidApiClientEnvironment:
    PRODUCTION: CandidApiClientEnvironment
    STAGING: CandidApiClientEnvironment
    CANDID_STAGING: CandidApiClientEnvironment

    def __init__(self, *, candid_api: str, pre_encounter: str):
        self.candid_api = candid_api
        self.pre_encounter = pre_encounter


CandidApiClientEnvironment.PRODUCTION = CandidApiClientEnvironment(
    candid_api="https://api.joincandidhealth.com", pre_encounter="https://pre-api.joincandidhealth.com"
)
CandidApiClientEnvironment.STAGING = CandidApiClientEnvironment(
    candid_api="https://api-staging.joincandidhealth.com", pre_encounter="https://pre-api-staging.joincandidhealth.com"
)
CandidApiClientEnvironment.CANDID_STAGING = CandidApiClientEnvironment(
    candid_api="https://staging-api.joincandidhealth.com", pre_encounter="https://staging-pre-api.joincandidhealth.com"
)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .organization_service_facility import OrganizationServiceFacility
from .organization_service_facility_create import OrganizationServiceFacilityCreate
from .organization_service_facility_id import OrganizationServiceFacilityId
from .organization_service_facility_page import OrganizationServiceFacilityPage
from .organization_service_facility_update import OrganizationServiceFacilityUpdate
from .service_facility_mode import ServiceFacilityMode
from .service_facility_operational_status import ServiceFacilityOperationalStatus
from .service_facility_physical_type import ServiceFacilityPhysicalType
from .service_facility_status import ServiceFacilityStatus
from .service_facility_type import ServiceFacilityType

__all__ = [
    "OrganizationServiceFacility",
    "OrganizationServiceFacilityCreate",
    "OrganizationServiceFacilityId",
    "OrganizationServiceFacilityPage",
    "OrganizationServiceFacilityUpdate",
    "ServiceFacilityMode",
    "ServiceFacilityOperationalStatus",
    "ServiceFacilityPhysicalType",
    "ServiceFacilityStatus",
    "ServiceFacilityType",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    Gender,
    IndividualBase,
    IndividualId,
    Patient,
    PatientBase,
    PatientClinicalTrialInfo,
    PatientClinicalTrialInfoCreate,
    PatientCreate,
    PatientNonInsurancePayerInfo,
    PatientNonInsurancePayerInfoCreate,
    PatientUpdate,
    PatientUpdateWithOptionalAddress,
    Subscriber,
    SubscriberBase,
    SubscriberCreate,
)

__all__ = [
    "Gender",
    "IndividualBase",
    "IndividualId",
    "Patient",
    "PatientBase",
    "PatientClinicalTrialInfo",
    "PatientClinicalTrialInfoCreate",
    "PatientCreate",
    "PatientNonInsurancePayerInfo",
    "PatientNonInsurancePayerInfoCreate",
    "PatientUpdate",
    "PatientUpdateWithOptionalAddress",
    "Subscriber",
    "SubscriberBase",
    "SubscriberCreate",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ClaimStatus(str, enum.Enum):
    BILLER_RECEIVED = "biller_received"
    CODED = "coded"
    SUBMITTED_TO_PAYER = "submitted_to_payer"
    MISSING_INFORMATION = "missing_information"
    NOT_BILLABLE = "not_billable"
    WAITING_FOR_PROVIDER = "waiting_for_provider"
    ERA_RECEIVED = "era_received"
    REJECTED = "rejected"
    DENIED = "denied"
    PAID = "paid"
    PAID_INCORRECTLY = "paid_incorrectly"
    FINALIZED_PAID = "finalized_paid"
    FINALIZED_DENIED = "finalized_denied"
    HELD_BY_CUSTOMER = "held_by_customer"
    ERA_REQUIRES_REVIEW = "era_requires_review"
    _UNKNOWN = "__CLAIMSTATUS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "ClaimStatus":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        biller_received: typing.Callable[[], T_Result],
        coded: typing.Callable[[], T_Result],
        submitted_to_payer: typing.Callable[[], T_Result],
        missing_information: typing.Callable[[], T_Result],
        not_billable: typing.Callable[[], T_Result],
        waiting_for_provider: typing.Callable[[], T_Result],
        era_received: typing.Callable[[], T_Result],
        rejected: typing.Callable[[], T_Result],
        denied: typing.Callable[[], T_Result],
        paid: typing.Callable[[], T_Result],
        paid_incorrectly: typing.Callable[[], T_Result],
        finalized_paid: typing.Callable[[], T_Result],
        finalized_denied: typing.Callable[[], T_Result],
        held_by_customer: typing.Callable[[], T_Result],
        era_requires_review: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is ClaimStatus.BILLER_RECEIVED:
            return biller_received()
        if self is ClaimStatus.CODED:
            return coded()
        if self is ClaimStatus.SUBMITTED_TO_PAYER:
            return submitted_to_payer()
        if self is ClaimStatus.MISSING_INFORMATION:
            return missing_information()
        if self is ClaimStatus.NOT_BILLABLE:
            return not_billable()
        if self is ClaimStatus.WAITING_FOR_PROVIDER:
            return waiting_for_provider()
        if self is ClaimStatus.ERA_RECEIVED:
            return era_received()
        if self is ClaimStatus.REJECTED:
            return rejected()
        if self is ClaimStatus.DENIED:
            return denied()
        if self is ClaimStatus.PAID:
            return paid()
        if self is ClaimStatus.PAID_INCORRECTLY:
            return paid_incorrectly()
        if self is ClaimStatus.FINALIZED_PAID:
            return finalized_paid()
        if self is ClaimStatus.FINALIZED_DENIED:
            return finalized_denied()
        if self is ClaimStatus.HELD_BY_CUSTOMER:
            return held_by_customer()
        if self is ClaimStatus.ERA_REQUIRES_REVIEW:
            return era_requires_review()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ...commons.types.email import Email
from ...commons.types.phone_number import PhoneNumber
from ...non_insurance_payers.resources.v_1.types.non_insurance_payer_id import NonInsurancePayerId
from .patient_base import PatientBase
from .patient_non_insurance_payer_info_create import PatientNonInsurancePayerInfoCreate


class PatientCreate(PatientBase):
    phone_numbers: typing.Optional[typing.List[PhoneNumber]] = None
    phone_consent: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Defaults to false
    """

    email: typing.Optional[Email] = None
    non_insurance_payers: typing.Optional[typing.List[NonInsurancePayerId]] = None
    non_insurance_payers_info: typing.Optional[typing.List[PatientNonInsurancePayerInfoCreate]] = None
    email_consent: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Defaults to false
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .coverage_association import CoverageAssociation
from .coverage_image_side import CoverageImageSide
from .image import Image
from .image_id import ImageId
from .image_status import ImageStatus
from .mutable_image import MutableImage
from .patient_association import PatientAssociation

__all__ = [
    "CoverageAssociation",
    "CoverageImageSide",
    "Image",
    "ImageId",
    "ImageStatus",
    "MutableImage",
    "PatientAssociation",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    AuthorizedSignatory,
    AuthorizedSignatoryUpdate,
    AuthorizedSignatoryUpdate_Remove,
    AuthorizedSignatoryUpdate_Set,
    Contract,
    ContractBase,
    ContractId,
    ContractInvalidExpirationDateError,
    ContractIsLinkedToFeeScheduleError,
    ContractSortField,
    ContractStatus,
    ContractWithProviders,
    ContractingProviderId,
    ContractsPage,
    DateUpdate,
    DateUpdate_Remove,
    DateUpdate_Set,
    InsuranceTypes,
    InsuranceTypes_AllApply,
    InsuranceTypes_NoneApply,
    InsuranceTypes_TheseApply,
    RegionsUpdate,
    RegionsUpdate_Remove,
    RegionsUpdate_Set,
    RenderingProviderid,
)
from .errors import ContractInvalidExpirationDateHttpError, ContractIsLinkedToFeeScheduleHttpError

__all__ = [
    "AuthorizedSignatory",
    "AuthorizedSignatoryUpdate",
    "AuthorizedSignatoryUpdate_Remove",
    "AuthorizedSignatoryUpdate_Set",
    "Contract",
    "ContractBase",
    "ContractId",
    "ContractInvalidExpirationDateError",
    "ContractInvalidExpirationDateHttpError",
    "ContractIsLinkedToFeeScheduleError",
    "ContractIsLinkedToFeeScheduleHttpError",
    "ContractSortField",
    "ContractStatus",
    "ContractWithProviders",
    "ContractingProviderId",
    "ContractsPage",
    "DateUpdate",
    "DateUpdate_Remove",
    "DateUpdate_Set",
    "InsuranceTypes",
    "InsuranceTypes_AllApply",
    "InsuranceTypes_NoneApply",
    "InsuranceTypes_TheseApply",
    "RegionsUpdate",
    "RegionsUpdate_Remove",
    "RegionsUpdate_Set",
    "RenderingProviderid",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .encounter_service_facility import EncounterServiceFacility
from .encounter_service_facility_base import EncounterServiceFacilityBase
from .encounter_service_facility_update import EncounterServiceFacilityUpdate
from .encounter_service_facility_update_with_optional_address import EncounterServiceFacilityUpdateWithOptionalAddress
from .service_facility_id import ServiceFacilityId

__all__ = [
    "EncounterServiceFacility",
    "EncounterServiceFacilityBase",
    "EncounterServiceFacilityUpdate",
    "EncounterServiceFacilityUpdateWithOptionalAddress",
    "ServiceFacilityId",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .lab_code_type import LabCodeType


class Lab(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.encounters.resources.v_4 import Lab, LabCodeType

    Lab(
        name="Genetic Health Labs",
        code="GH12345",
        code_type=LabCodeType.QUEST,
    )
    """

    name: str
    code: typing.Optional[str] = None
    code_type: typing.Optional[LabCodeType] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import InsurancePayment, InsurancePaymentId, InsurancePaymentSortField, InsurancePaymentsPage

__all__ = ["InsurancePayment", "InsurancePaymentId", "InsurancePaymentSortField", "InsurancePaymentsPage"]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class TagColorEnum(str, enum.Enum):
    BLACK = "black"
    WHITE = "white"
    GRAY = "gray"
    RED = "red"
    YELLOW = "yellow"
    GREEN = "green"
    BLUE = "blue"
    INDIGO = "indigo"
    PURPLE = "purple"
    PINK = "pink"
    _UNKNOWN = "__TAGCOLORENUM_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "TagColorEnum":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        black: typing.Callable[[], T_Result],
        white: typing.Callable[[], T_Result],
        gray: typing.Callable[[], T_Result],
        red: typing.Callable[[], T_Result],
        yellow: typing.Callable[[], T_Result],
        green: typing.Callable[[], T_Result],
        blue: typing.Callable[[], T_Result],
        indigo: typing.Callable[[], T_Result],
        purple: typing.Callable[[], T_Result],
        pink: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is TagColorEnum.BLACK:
            return black()
        if self is TagColorEnum.WHITE:
            return white()
        if self is TagColorEnum.GRAY:
            return gray()
        if self is TagColorEnum.RED:
            return red()
        if self is TagColorEnum.YELLOW:
            return yellow()
        if self is TagColorEnum.GREEN:
            return green()
        if self is TagColorEnum.BLUE:
            return blue()
        if self is TagColorEnum.INDIGO:
            return indigo()
        if self is TagColorEnum.PURPLE:
            return purple()
        if self is TagColorEnum.PINK:
            return pink()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .contract_invalid_expiration_date_http_error import ContractInvalidExpirationDateHttpError
from .contract_is_linked_to_fee_schedule_http_error import ContractIsLinkedToFeeScheduleHttpError

__all__ = ["ContractInvalidExpirationDateHttpError", "ContractIsLinkedToFeeScheduleHttpError"]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.npi import Npi
from .....commons.types.tax_id import TaxId


class PayeeIdentifier_Npi(UniversalBaseModel):
    value: Npi
    type: typing.Literal["npi"] = "npi"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class PayeeIdentifier_Tin(UniversalBaseModel):
    value: TaxId
    type: typing.Literal["tin"] = "tin"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


class PayeeIdentifier_CmsPlanId(UniversalBaseModel):
    value: str
    type: typing.Literal["cms_plan_id"] = "cms_plan_id"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True


PayeeIdentifier = typing.Union[PayeeIdentifier_Npi, PayeeIdentifier_Tin, PayeeIdentifier_CmsPlanId]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .gender import Gender
from .individual_base import IndividualBase
from .individual_id import IndividualId
from .patient import Patient
from .patient_base import PatientBase
from .patient_clinical_trial_info import PatientClinicalTrialInfo
from .patient_clinical_trial_info_create import PatientClinicalTrialInfoCreate
from .patient_create import PatientCreate
from .patient_non_insurance_payer_info import PatientNonInsurancePayerInfo
from .patient_non_insurance_payer_info_create import PatientNonInsurancePayerInfoCreate
from .patient_update import PatientUpdate
from .patient_update_with_optional_address import PatientUpdateWithOptionalAddress
from .subscriber import Subscriber
from .subscriber_base import SubscriberBase
from .subscriber_create import SubscriberCreate

__all__ = [
    "Gender",
    "IndividualBase",
    "IndividualId",
    "Patient",
    "PatientBase",
    "PatientClinicalTrialInfo",
    "PatientClinicalTrialInfoCreate",
    "PatientCreate",
    "PatientNonInsurancePayerInfo",
    "PatientNonInsurancePayerInfoCreate",
    "PatientUpdate",
    "PatientUpdateWithOptionalAddress",
    "Subscriber",
    "SubscriberBase",
    "SubscriberCreate",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....payers.resources.v_3.types.payer_uuid import PayerUuid
from .rate_entry import RateEntry


class ValidationError_OverlappingRateEntries(UniversalBaseModel):
    type: typing.Literal["overlapping_rate_entries"] = "overlapping_rate_entries"
    message: str
    rate_a: RateEntry
    rate_b: RateEntry

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ValidationError_VersionConflict(UniversalBaseModel):
    type: typing.Literal["version_conflict"] = "version_conflict"
    entity_name: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ValidationError_OrganizationProviderNotFound(UniversalBaseModel):
    type: typing.Literal["organization_provider_not_found"] = "organization_provider_not_found"
    id: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ValidationError_DuplicateRate(UniversalBaseModel):
    type: typing.Literal["duplicate_rate"] = "duplicate_rate"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ValidationError_EmptyEntries(UniversalBaseModel):
    type: typing.Literal["empty_entries"] = "empty_entries"

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ValidationError_PayerPlanGroupNotFound(UniversalBaseModel):
    type: typing.Literal["payer_plan_group_not_found"] = "payer_plan_group_not_found"
    id: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ValidationError_PayerPlanGroupDoesNotMatchRatePayer(UniversalBaseModel):
    type: typing.Literal["payer_plan_group_does_not_match_rate_payer"] = "payer_plan_group_does_not_match_rate_payer"
    rate_payer_uuid: PayerUuid
    payer_plan_group_payer_uuid: PayerUuid

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ValidationError = typing.Union[
    ValidationError_OverlappingRateEntries,
    ValidationError_VersionConflict,
    ValidationError_OrganizationProviderNotFound,
    ValidationError_DuplicateRate,
    ValidationError_EmptyEntries,
    ValidationError_PayerPlanGroupNotFound,
    ValidationError_PayerPlanGroupDoesNotMatchRatePayer,
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2
from .....billing_notes.resources.v_2.types.billing_note_base import BillingNoteBase
from .....claim_submission.resources.v_1.types.external_claim_submission_create import ExternalClaimSubmissionCreate
from .....commons.types.street_address_short_zip_optional import StreetAddressShortZipOptional
from .....diagnoses.types.diagnosis_create import DiagnosisCreate
from .....encounter_providers.resources.v_2.types.billing_provider_update_with_optional_address import (
    BillingProviderUpdateWithOptionalAddress,
)
from .....encounter_providers.resources.v_2.types.initial_referring_provider_update_with_optional_address import (
    InitialReferringProviderUpdateWithOptionalAddress,
)
from .....encounter_providers.resources.v_2.types.referring_provider_update_with_optional_address import (
    ReferringProviderUpdateWithOptionalAddress,
)
from .....encounter_providers.resources.v_2.types.rendering_provider_update_with_optional_address import (
    RenderingProviderUpdateWithOptionalAddress,
)
from .....encounter_providers.resources.v_2.types.supervising_provider_update_with_optional_address import (
    SupervisingProviderUpdateWithOptionalAddress,
)
from .....encounters.resources.v_4.types.encounter_optional import EncounterOptional
from .....encounters.resources.v_4.types.intervention import Intervention
from .....encounters.resources.v_4.types.patient_history_category import PatientHistoryCategory
from .....individual.types.patient_update_with_optional_address import PatientUpdateWithOptionalAddress
from .....service_facility.types.encounter_service_facility_update_with_optional_address import (
    EncounterServiceFacilityUpdateWithOptionalAddress,
)
from .....service_lines.resources.v_2.types.service_line_create import ServiceLineCreate


class ChargeCaptureData(EncounterOptional):
    diagnoses: typing.Optional[typing.List[DiagnosisCreate]] = pydantic.Field(default=None)
    """
    Ideally, this field should contain no more than 12 diagnoses. However, more diagnoses
    may be submitted at this time, and coders will later prioritize the 12 that will be
    submitted to the payor.
    """

    interventions: typing.Optional[typing.List[Intervention]] = None
    external_claim_submission: typing.Optional[ExternalClaimSubmissionCreate] = pydantic.Field(default=None)
    """
    ***This field is in beta.***
    To be included for claims that have been submitted outside of Candid.
    Candid supports posting remits and payments to these claims and working them in-platform (e.g. editing, resubmitting).
    """

    service_lines: typing.Optional[typing.List[ServiceLineCreate]] = pydantic.Field(default=None)
    """
    Each service line must be linked to a diagnosis. Concretely,
    `service_line.diagnosis_pointers`must contain at least one entry which should be
    in bounds of the diagnoses list field.
    """

    patient_histories: typing.Optional[typing.List[PatientHistoryCategory]] = None
    billing_notes: typing.Optional[typing.List[BillingNoteBase]] = pydantic.Field(default=None)
    """
    Spot to store misc, human-readable, notes about this encounter to be
    used in the billing process.
    """

    patient: typing.Optional[PatientUpdateWithOptionalAddress] = pydantic.Field(default=None)
    """
    Contains the identification information of the individual receiving medical services.
    """

    service_facility: typing.Optional[EncounterServiceFacilityUpdateWithOptionalAddress] = pydantic.Field(default=None)
    """
    Encounter Service facility is typically the location a medical service was rendered, such as a provider office or hospital. For telehealth, service facility can represent the provider's location when the service was delivered (e.g., home), or the location where an in-person visit would have taken place, whichever is easier to identify. If the provider is in-network, service facility may be defined in payer contracts. Box 32 on the CMS-1500 claim form. Note that for an in-network claim to be successfully adjudicated, the service facility address listed on claims must match what was provided to the payer during the credentialing process.
    """

    rendering_provider: typing.Optional[RenderingProviderUpdateWithOptionalAddress] = pydantic.Field(default=None)
    """
    The rendering provider is the practitioner -- physician, nurse practitioner, etc. -- performing the service.
    For telehealth services, the rendering provider performs the visit, asynchronous communication, or other service. The rendering provider address should generally be the same as the service facility address.
    """

    initial_referring_provider: typing.Optional[InitialReferringProviderUpdateWithOptionalAddress] = pydantic.Field(
        default=None
    )
    """
    The second iteration of Loop ID-2310. Use code "P3 - Primary Care Provider" in this loop to
    indicate the initial referral from the primary care provider or whatever provider wrote the initial referral for this patient's episode of care being billed/reported in this transaction.
    """

    referring_provider: typing.Optional[ReferringProviderUpdateWithOptionalAddress] = pydantic.Field(default=None)
    """
    The final provider who referred the services that were rendered.
    All physicians who order services or refer Medicare beneficiaries must
    report this data.
    """

    supervising_provider: typing.Optional[SupervisingProviderUpdateWithOptionalAddress] = pydantic.Field(default=None)
    """
    Required when the rendering provider is supervised by a physician. If not required by this implementation guide, do not send.
    """

    billing_provider: typing.Optional[BillingProviderUpdateWithOptionalAddress] = pydantic.Field(default=None)
    """
    The billing provider is the provider or business entity submitting the claim. Billing provider may be, but is not necessarily, the same person/NPI as the rendering provider. From a payer's perspective, this represents the person or entity being reimbursed. When a contract exists with the target payer, the billing provider should be the entity contracted with the payer. In some circumstances, this will be an individual provider. In that case, submit that provider's NPI and the tax ID (TIN) that the provider gave to the payer during contracting. In other cases, the billing entity will be a medical group. If so, submit the group NPI and the group's tax ID. Box 33 on the CMS-1500 claim form.
    """

    pay_to_address: typing.Optional[StreetAddressShortZipOptional] = pydantic.Field(default=None)
    """
    Specifies the address to which payments for the claim should be sent.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .invoice_destination import InvoiceDestination


class InvoiceDestinationMetadata(UniversalBaseModel):
    invoice_destination: InvoiceDestination = pydantic.Field()
    """
    Defines which third-party service this invoice was created in
    """

    source_id: str = pydantic.Field()
    """
    The id of the invoice in the third-party service
    """

    source_customer_id: str = pydantic.Field()
    """
    The id of the customer that the invoice is attributed to in the third-party service
    """

    destination_status: typing.Optional[str] = pydantic.Field(default=None)
    """
    The status of the invoice in the third-party service
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawFeeSchedulesClient, RawFeeSchedulesClient
from .resources.v_3.client import AsyncV3Client, V3Client


class FeeSchedulesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawFeeSchedulesClient(client_wrapper=client_wrapper)
        self.v_3 = V3Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawFeeSchedulesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawFeeSchedulesClient
        """
        return self._raw_client


class AsyncFeeSchedulesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawFeeSchedulesClient(client_wrapper=client_wrapper)
        self.v_3 = AsyncV3Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawFeeSchedulesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawFeeSchedulesClient
        """
        return self._raw_client



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import ExternalPaymentAccountConfig, ExternalPaymentAccountConfigPage

__all__ = ["ExternalPaymentAccountConfig", "ExternalPaymentAccountConfigPage"]



# This file was auto-generated by Fern from our API Definition.

import typing

from ..... import core
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....commons.types.encounter_id import EncounterId
from .raw_client import AsyncRawV1Client, RawV1Client
from .types.attachment_id import AttachmentId
from .types.encounter_attachment import EncounterAttachment
from .types.encounter_attachment_type import EncounterAttachmentType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV1Client
        """
        return self._raw_client

    def get(
        self, encounter_id: EncounterId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[EncounterAttachment]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[EncounterAttachment]

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.encounter_attachments.v_1.get(
            encounter_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.get(encounter_id, request_options=request_options)
        return _response.data

    def create(
        self,
        encounter_id: EncounterId,
        *,
        attachment_file: core.File,
        attachment_type: EncounterAttachmentType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AttachmentId:
        """
        Uploads a file to the encounter. The file will be stored in the
        encounter's attachments.

        Parameters
        ----------
        encounter_id : EncounterId

        attachment_file : core.File
            See core.File for more documentation

        attachment_type : EncounterAttachmentType

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttachmentId
        """
        _response = self._raw_client.create(
            encounter_id,
            attachment_file=attachment_file,
            attachment_type=attachment_type,
            request_options=request_options,
        )
        return _response.data

    def delete(
        self,
        encounter_id: EncounterId,
        *,
        attachment_id: AttachmentId,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        attachment_id : AttachmentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.encounter_attachments.v_1.delete(
            encounter_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            attachment_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
        )
        """
        _response = self._raw_client.delete(encounter_id, attachment_id=attachment_id, request_options=request_options)
        return _response.data


class AsyncV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV1Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV1Client
        """
        return self._raw_client

    async def get(
        self, encounter_id: EncounterId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> typing.List[EncounterAttachment]:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[EncounterAttachment]

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.encounter_attachments.v_1.get(
                encounter_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(encounter_id, request_options=request_options)
        return _response.data

    async def create(
        self,
        encounter_id: EncounterId,
        *,
        attachment_file: core.File,
        attachment_type: EncounterAttachmentType,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AttachmentId:
        """
        Uploads a file to the encounter. The file will be stored in the
        encounter's attachments.

        Parameters
        ----------
        encounter_id : EncounterId

        attachment_file : core.File
            See core.File for more documentation

        attachment_type : EncounterAttachmentType

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AttachmentId
        """
        _response = await self._raw_client.create(
            encounter_id,
            attachment_file=attachment_file,
            attachment_type=attachment_type,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self,
        encounter_id: EncounterId,
        *,
        attachment_id: AttachmentId,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Parameters
        ----------
        encounter_id : EncounterId

        attachment_id : AttachmentId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.encounter_attachments.v_1.delete(
                encounter_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                attachment_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(
            encounter_id, attachment_id=attachment_id, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .intake_follow_up_id import IntakeFollowUpId


class IntakeFollowUp(UniversalBaseModel):
    """
    Examples
    --------
    from candid.resources.encounters.resources.v_4 import IntakeFollowUp

    IntakeFollowUp(
        id="4F3D57F9-AC94-49D6-87E4-E804B709917A",
        text="Do you have any allergies?",
        response="No allergies",
    )
    """

    id: IntakeFollowUpId
    text: str
    response: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class OrganizationProviderSortOptions(str, enum.Enum):
    PROVIDER_NAME_ASC = "provider_name:asc"
    PROVIDER_NAME_DESC = "provider_name:desc"
    NPI_ASC = "npi:asc"
    NPI_DESC = "npi:desc"
    _UNKNOWN = "__ORGANIZATIONPROVIDERSORTOPTIONS_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "OrganizationProviderSortOptions":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        provider_name_asc: typing.Callable[[], T_Result],
        provider_name_desc: typing.Callable[[], T_Result],
        npi_asc: typing.Callable[[], T_Result],
        npi_desc: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is OrganizationProviderSortOptions.PROVIDER_NAME_ASC:
            return provider_name_asc()
        if self is OrganizationProviderSortOptions.PROVIDER_NAME_DESC:
            return provider_name_desc()
        if self is OrganizationProviderSortOptions.NPI_ASC:
            return npi_asc()
        if self is OrganizationProviderSortOptions.NPI_DESC:
            return npi_desc()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .key_with_name_already_exists_error import KeyWithNameAlreadyExistsError
from .schema import Schema
from .schema_field import SchemaField
from .schema_get_multi_response import SchemaGetMultiResponse
from .schema_instance import SchemaInstance
from .schema_validation_error import (
    SchemaValidationError,
    SchemaValidationError_KeyNameAlreadyExists,
    SchemaValidationError_SchemaNameAlreadyExists,
)
from .schema_validation_failure import SchemaValidationFailure
from .schema_with_name_already_exists_error import SchemaWithNameAlreadyExistsError

__all__ = [
    "KeyWithNameAlreadyExistsError",
    "Schema",
    "SchemaField",
    "SchemaGetMultiResponse",
    "SchemaInstance",
    "SchemaValidationError",
    "SchemaValidationError_KeyNameAlreadyExists",
    "SchemaValidationError_SchemaNameAlreadyExists",
    "SchemaValidationFailure",
    "SchemaWithNameAlreadyExistsError",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    Authorization,
    AuthorizationUnit,
    Contact,
    DoNotInvoiceReason,
    ExternalProvenance,
    FilingOrder,
    Guarantor,
    MaritalStatus,
    MutablePatient,
    MutablePatientWithMrn,
    Patient,
    PatientPage,
    PatientSortField,
    Referral,
)

__all__ = [
    "Authorization",
    "AuthorizationUnit",
    "Contact",
    "DoNotInvoiceReason",
    "ExternalProvenance",
    "FilingOrder",
    "Guarantor",
    "MaritalStatus",
    "MutablePatient",
    "MutablePatientWithMrn",
    "Patient",
    "PatientPage",
    "PatientSortField",
    "Referral",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .......core.api_error import ApiError
from .......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .......core.http_response import AsyncHttpResponse, HttpResponse
from .......core.jsonable_encoder import jsonable_encoder
from .......core.pydantic_utilities import parse_obj_as
from .......core.request_options import RequestOptions
from ....common.errors.not_found_error import NotFoundError
from ....common.errors.version_conflict_error import VersionConflictError
from ....common.types.error_base_4_xx import ErrorBase4Xx
from ....common.types.version_conflict_error_body import VersionConflictErrorBody
from .types.image import Image
from .types.image_id import ImageId
from .types.mutable_image import MutableImage

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self, *, request: MutableImage, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Image]:
        """
        Adds an image.  VersionConflictError is returned if a front or back of this coverage already exists.

        Parameters
        ----------
        request : MutableImage

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Image]
        """
        _response = self._client_wrapper.httpx_client.request(
            "images/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Image,
                parse_obj_as(
                    type_=Image,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(self, id: ImageId, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Image]:
        """
        Gets an image by imageId.

        Parameters
        ----------
        id : ImageId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Image]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"images/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Image,
                parse_obj_as(
                    type_=Image,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        id: ImageId,
        version: str,
        *,
        request: MutableImage,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Image]:
        """
        Updates an Image.  The path must contain the most recent version to prevent races.

        Parameters
        ----------
        id : ImageId

        version : str

        request : MutableImage

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Image]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"images/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Image,
                parse_obj_as(
                    type_=Image,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def deactivate(
        self, id: ImageId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Sets an Image as deactivated.  The path must contain the most recent version to prevent races.

        Parameters
        ----------
        id : ImageId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"images/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_multi(
        self,
        *,
        patient_id: typing.Optional[str] = None,
        coverage_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[Image]]:
        """
        Searches for images that match the query parameters.

        Parameters
        ----------
        patient_id : typing.Optional[str]

        coverage_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[Image]]
        """
        _response = self._client_wrapper.httpx_client.request(
            "images/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "patient_id": patient_id,
                "coverage_id": coverage_id,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Image],
                parse_obj_as(
                    type_=typing.List[Image],  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self, *, request: MutableImage, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Image]:
        """
        Adds an image.  VersionConflictError is returned if a front or back of this coverage already exists.

        Parameters
        ----------
        request : MutableImage

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Image]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "images/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Image,
                parse_obj_as(
                    type_=Image,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: ImageId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Image]:
        """
        Gets an image by imageId.

        Parameters
        ----------
        id : ImageId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Image]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"images/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Image,
                parse_obj_as(
                    type_=Image,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        id: ImageId,
        version: str,
        *,
        request: MutableImage,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Image]:
        """
        Updates an Image.  The path must contain the most recent version to prevent races.

        Parameters
        ----------
        id : ImageId

        version : str

        request : MutableImage

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Image]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"images/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Image,
                parse_obj_as(
                    type_=Image,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def deactivate(
        self, id: ImageId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Sets an Image as deactivated.  The path must contain the most recent version to prevent races.

        Parameters
        ----------
        id : ImageId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"images/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "NotFoundError":
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        ErrorBase4Xx,
                        parse_obj_as(
                            type_=ErrorBase4Xx,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "VersionConflictError":
                raise VersionConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        VersionConflictErrorBody,
                        parse_obj_as(
                            type_=VersionConflictErrorBody,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_multi(
        self,
        *,
        patient_id: typing.Optional[str] = None,
        coverage_id: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[Image]]:
        """
        Searches for images that match the query parameters.

        Parameters
        ----------
        patient_id : typing.Optional[str]

        coverage_id : typing.Optional[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[Image]]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "images/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            params={
                "patient_id": patient_id,
                "coverage_id": coverage_id,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                typing.List[Image],
                parse_obj_as(
                    type_=typing.List[Image],  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
from json.decoder import JSONDecodeError

from .....core.api_error import ApiError
from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.datetime_utils import serialize_datetime
from .....core.http_response import AsyncHttpResponse, HttpResponse
from .....core.jsonable_encoder import jsonable_encoder
from .....core.pydantic_utilities import parse_obj_as
from .....core.request_options import RequestOptions
from ....commons.errors.entity_not_found_error import EntityNotFoundError
from ....commons.errors.unauthorized_error import UnauthorizedError
from ....commons.errors.unprocessable_entity_error import UnprocessableEntityError
from ....commons.types.encounter_id import EncounterId
from ....commons.types.entity_not_found_error_message import EntityNotFoundErrorMessage
from ....commons.types.page_token import PageToken
from ....commons.types.task_id import TaskId
from ....commons.types.unauthorized_error_message import UnauthorizedErrorMessage
from ....commons.types.unprocessable_entity_error_message import UnprocessableEntityErrorMessage
from ....commons.types.user_id import UserId
from ..commons.types.task_status import TaskStatus
from ..commons.types.task_type import TaskType
from .errors.task_updated_to_deprecated_status_error import TaskUpdatedToDeprecatedStatusError
from .types.task import Task
from .types.task_actions import TaskActions
from .types.task_create_v_3 import TaskCreateV3
from .types.task_page import TaskPage
from .types.task_sort_options import TaskSortOptions
from .types.task_update_v_3 import TaskUpdateV3
from .types.task_updated_to_deprecated_status_error_type import TaskUpdatedToDeprecatedStatusErrorType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV3Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get_actions(
        self, task_id: TaskId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[TaskActions]:
        """
        Parameters
        ----------
        task_id : TaskId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TaskActions]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/tasks/v3/{jsonable_encoder(task_id)}/actions",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                TaskActions,
                parse_obj_as(
                    type_=TaskActions,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        status: typing.Optional[TaskStatus] = None,
        task_type: typing.Optional[TaskType] = None,
        categories: typing.Optional[str] = None,
        updated_since: typing.Optional[dt.datetime] = None,
        encounter_id: typing.Optional[EncounterId] = None,
        search_term: typing.Optional[str] = None,
        assigned_to_id: typing.Optional[UserId] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        billing_provider_npi: typing.Optional[str] = None,
        sort: typing.Optional[TaskSortOptions] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[TaskPage]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Defaults to 100

        page_token : typing.Optional[PageToken]

        status : typing.Optional[TaskStatus]

        task_type : typing.Optional[TaskType]

        categories : typing.Optional[str]
            Only return tasks with categories that match one in this comma-separated list.

        updated_since : typing.Optional[dt.datetime]
            Only return tasks updated on or after this date-time

        encounter_id : typing.Optional[EncounterId]
            Only return tasks associated with this encounter

        search_term : typing.Optional[str]
            Query tasks by encounter_id, claim_id, task_id, or external_id

        assigned_to_id : typing.Optional[UserId]
            Only return tasks assigned to this user

        date_of_service_min : typing.Optional[dt.date]
            The minimum date of service for the linked encounter

        date_of_service_max : typing.Optional[dt.date]
            The maximum date of service for the linked encounter

        billing_provider_npi : typing.Optional[str]
            The NPI of the billing provider associated with the task's claim

        sort : typing.Optional[TaskSortOptions]
            Defaults to updated_at:desc

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[TaskPage]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/tasks/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "page_token": page_token,
                "status": status,
                "task_type": task_type,
                "categories": categories,
                "updated_since": serialize_datetime(updated_since) if updated_since is not None else None,
                "encounter_id": encounter_id,
                "search_term": search_term,
                "assigned_to_id": assigned_to_id,
                "date_of_service_min": str(date_of_service_min) if date_of_service_min is not None else None,
                "date_of_service_max": str(date_of_service_max) if date_of_service_max is not None else None,
                "billing_provider_npi": billing_provider_npi,
                "sort": sort,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                TaskPage,
                parse_obj_as(
                    type_=TaskPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(self, task_id: TaskId, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Task]:
        """
        Parameters
        ----------
        task_id : TaskId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Task]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/tasks/v3/{jsonable_encoder(task_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Task,
                parse_obj_as(
                    type_=Task,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, *, request: TaskCreateV3, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Task]:
        """
        Parameters
        ----------
        request : TaskCreateV3

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Task]
        """
        _response = self._client_wrapper.httpx_client.request(
            "api/tasks/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Task,
                parse_obj_as(
                    type_=Task,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self, task_id: TaskId, *, request: TaskUpdateV3, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Task]:
        """
        Parameters
        ----------
        task_id : TaskId

        request : TaskUpdateV3

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Task]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"api/tasks/v3/{jsonable_encoder(task_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Task,
                parse_obj_as(
                    type_=Task,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "TaskUpdatedToDeprecatedStatusError":
                raise TaskUpdatedToDeprecatedStatusError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TaskUpdatedToDeprecatedStatusErrorType,
                        parse_obj_as(
                            type_=TaskUpdatedToDeprecatedStatusErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV3Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get_actions(
        self, task_id: TaskId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[TaskActions]:
        """
        Parameters
        ----------
        task_id : TaskId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TaskActions]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/tasks/v3/{jsonable_encoder(task_id)}/actions",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                TaskActions,
                parse_obj_as(
                    type_=TaskActions,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        page_token: typing.Optional[PageToken] = None,
        status: typing.Optional[TaskStatus] = None,
        task_type: typing.Optional[TaskType] = None,
        categories: typing.Optional[str] = None,
        updated_since: typing.Optional[dt.datetime] = None,
        encounter_id: typing.Optional[EncounterId] = None,
        search_term: typing.Optional[str] = None,
        assigned_to_id: typing.Optional[UserId] = None,
        date_of_service_min: typing.Optional[dt.date] = None,
        date_of_service_max: typing.Optional[dt.date] = None,
        billing_provider_npi: typing.Optional[str] = None,
        sort: typing.Optional[TaskSortOptions] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[TaskPage]:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Defaults to 100

        page_token : typing.Optional[PageToken]

        status : typing.Optional[TaskStatus]

        task_type : typing.Optional[TaskType]

        categories : typing.Optional[str]
            Only return tasks with categories that match one in this comma-separated list.

        updated_since : typing.Optional[dt.datetime]
            Only return tasks updated on or after this date-time

        encounter_id : typing.Optional[EncounterId]
            Only return tasks associated with this encounter

        search_term : typing.Optional[str]
            Query tasks by encounter_id, claim_id, task_id, or external_id

        assigned_to_id : typing.Optional[UserId]
            Only return tasks assigned to this user

        date_of_service_min : typing.Optional[dt.date]
            The minimum date of service for the linked encounter

        date_of_service_max : typing.Optional[dt.date]
            The maximum date of service for the linked encounter

        billing_provider_npi : typing.Optional[str]
            The NPI of the billing provider associated with the task's claim

        sort : typing.Optional[TaskSortOptions]
            Defaults to updated_at:desc

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[TaskPage]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/tasks/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            params={
                "limit": limit,
                "page_token": page_token,
                "status": status,
                "task_type": task_type,
                "categories": categories,
                "updated_since": serialize_datetime(updated_since) if updated_since is not None else None,
                "encounter_id": encounter_id,
                "search_term": search_term,
                "assigned_to_id": assigned_to_id,
                "date_of_service_min": str(date_of_service_min) if date_of_service_min is not None else None,
                "date_of_service_max": str(date_of_service_max) if date_of_service_max is not None else None,
                "billing_provider_npi": billing_provider_npi,
                "sort": sort,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                TaskPage,
                parse_obj_as(
                    type_=TaskPage,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "UnprocessableEntityError":
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnprocessableEntityErrorMessage,
                        parse_obj_as(
                            type_=UnprocessableEntityErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, task_id: TaskId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Task]:
        """
        Parameters
        ----------
        task_id : TaskId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Task]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/tasks/v3/{jsonable_encoder(task_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Task,
                parse_obj_as(
                    type_=Task,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, *, request: TaskCreateV3, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Task]:
        """
        Parameters
        ----------
        request : TaskCreateV3

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Task]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "api/tasks/v3",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Task,
                parse_obj_as(
                    type_=Task,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self, task_id: TaskId, *, request: TaskUpdateV3, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Task]:
        """
        Parameters
        ----------
        task_id : TaskId

        request : TaskUpdateV3

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Task]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"api/tasks/v3/{jsonable_encoder(task_id)}",
            base_url=self._client_wrapper.get_environment().candid_api,
            method="PATCH",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Task,
                parse_obj_as(
                    type_=Task,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "EntityNotFoundError":
                raise EntityNotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        EntityNotFoundErrorMessage,
                        parse_obj_as(
                            type_=EntityNotFoundErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "UnauthorizedError":
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        UnauthorizedErrorMessage,
                        parse_obj_as(
                            type_=UnauthorizedErrorMessage,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
            if _response_json["errorName"] == "TaskUpdatedToDeprecatedStatusError":
                raise TaskUpdatedToDeprecatedStatusError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        TaskUpdatedToDeprecatedStatusErrorType,
                        parse_obj_as(
                            type_=TaskUpdatedToDeprecatedStatusErrorType,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    ),
                )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .attachment_id import AttachmentId
from .base_attachment import BaseAttachment
from .encounter_attachment import EncounterAttachment
from .encounter_attachment_type import EncounterAttachmentType

__all__ = ["AttachmentId", "BaseAttachment", "EncounterAttachment", "EncounterAttachmentType"]



# This file was auto-generated by Fern from our API Definition.

import typing

from .....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .....core.request_options import RequestOptions
from ....commons.types.page_token import PageToken
from ..v_2.types.organization_provider_id import OrganizationProviderId
from ..v_2.types.organization_provider_sort_options import OrganizationProviderSortOptions
from .raw_client import AsyncRawV3Client, RawV3Client
from .types.organization_provider_create_v_2 import OrganizationProviderCreateV2
from .types.organization_provider_page_v_2 import OrganizationProviderPageV2
from .types.organization_provider_update_v_2 import OrganizationProviderUpdateV2
from .types.organization_provider_v_2 import OrganizationProviderV2

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V3Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawV3Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawV3Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawV3Client
        """
        return self._raw_client

    def get(
        self,
        organization_provider_id: OrganizationProviderId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationProviderV2:
        """
        Parameters
        ----------
        organization_provider_id : OrganizationProviderId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderV2

        Examples
        --------
        import uuid

        from candid import CandidApiClient

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_providers.v_3.get(
            organization_provider_id=uuid.UUID(
                "965a563a-0285-4910-9569-e3739c0f6eab",
            ),
        )
        """
        _response = self._raw_client.get(organization_provider_id, request_options=request_options)
        return _response.data

    def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        search_term: typing.Optional[str] = None,
        npi: typing.Optional[str] = None,
        is_rendering: typing.Optional[bool] = None,
        is_billing: typing.Optional[bool] = None,
        page_token: typing.Optional[PageToken] = None,
        sort: typing.Optional[OrganizationProviderSortOptions] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationProviderPageV2:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Limit the number of results returned. Defaults to 100.

        search_term : typing.Optional[str]
            Filter to a name or a part of a name.

        npi : typing.Optional[str]
            Filter to a specific NPI.

        is_rendering : typing.Optional[bool]
            Filter to only rendering providers.

        is_billing : typing.Optional[bool]
            Filter to only billing providers.

        page_token : typing.Optional[PageToken]
            The page token to continue paging through a previous request.

        sort : typing.Optional[OrganizationProviderSortOptions]
            Defaults to PROVIDER_NAME_ASC.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderPageV2

        Examples
        --------
        from candid import CandidApiClient
        from candid.resources.organization_providers.resources.v_2 import (
            OrganizationProviderSortOptions,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_providers.v_3.get_multi(
            limit=100,
            search_term="john",
            npi="1234567890",
            is_rendering=True,
            is_billing=True,
            page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
            sort=OrganizationProviderSortOptions.PROVIDER_NAME_ASC,
        )
        """
        _response = self._raw_client.get_multi(
            limit=limit,
            search_term=search_term,
            npi=npi,
            is_rendering=is_rendering,
            is_billing=is_billing,
            page_token=page_token,
            sort=sort,
            request_options=request_options,
        )
        return _response.data

    def create(
        self, *, request: OrganizationProviderCreateV2, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationProviderV2:
        """
        Parameters
        ----------
        request : OrganizationProviderCreateV2

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderV2

        Examples
        --------
        from candid import CandidApiClient
        from candid.resources.commons import State
        from candid.resources.identifiers import (
            IdentifierCode,
            IdentifierCreate,
            IdentifierValue_MedicareProviderIdentifier,
        )
        from candid.resources.organization_providers.resources.v_2 import (
            LicenseType,
            ProviderType,
        )
        from candid.resources.organization_providers.resources.v_3 import (
            OrganizationProviderCreateV2,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_providers.v_3.create(
            request=OrganizationProviderCreateV2(
                npi="npi",
                is_rendering=True,
                is_billing=True,
                provider_type=ProviderType.INDIVIDUAL,
                license_type=LicenseType.MD,
                qualifications=[
                    IdentifierCreate(
                        identifier_code=IdentifierCode.MCR,
                        identifier_value=IdentifierValue_MedicareProviderIdentifier(
                            state=State.AA,
                            provider_number="provider_number",
                        ),
                    ),
                    IdentifierCreate(
                        identifier_code=IdentifierCode.MCR,
                        identifier_value=IdentifierValue_MedicareProviderIdentifier(
                            state=State.AA,
                            provider_number="provider_number",
                        ),
                    ),
                ],
            ),
        )
        """
        _response = self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    def update(
        self,
        organization_provider_id: OrganizationProviderId,
        *,
        request: OrganizationProviderUpdateV2,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationProviderV2:
        """
        Parameters
        ----------
        organization_provider_id : OrganizationProviderId

        request : OrganizationProviderUpdateV2

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderV2

        Examples
        --------
        import uuid

        from candid import CandidApiClient
        from candid.resources.organization_providers.resources.v_3 import (
            OrganizationProviderUpdateV2,
        )

        client = CandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )
        client.organization_providers.v_3.update(
            organization_provider_id=uuid.UUID(
                "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
            ),
            request=OrganizationProviderUpdateV2(),
        )
        """
        _response = self._raw_client.update(organization_provider_id, request=request, request_options=request_options)
        return _response.data


class AsyncV3Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawV3Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawV3Client:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawV3Client
        """
        return self._raw_client

    async def get(
        self,
        organization_provider_id: OrganizationProviderId,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationProviderV2:
        """
        Parameters
        ----------
        organization_provider_id : OrganizationProviderId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderV2

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.organization_providers.v_3.get(
                organization_provider_id=uuid.UUID(
                    "965a563a-0285-4910-9569-e3739c0f6eab",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(organization_provider_id, request_options=request_options)
        return _response.data

    async def get_multi(
        self,
        *,
        limit: typing.Optional[int] = None,
        search_term: typing.Optional[str] = None,
        npi: typing.Optional[str] = None,
        is_rendering: typing.Optional[bool] = None,
        is_billing: typing.Optional[bool] = None,
        page_token: typing.Optional[PageToken] = None,
        sort: typing.Optional[OrganizationProviderSortOptions] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationProviderPageV2:
        """
        Parameters
        ----------
        limit : typing.Optional[int]
            Limit the number of results returned. Defaults to 100.

        search_term : typing.Optional[str]
            Filter to a name or a part of a name.

        npi : typing.Optional[str]
            Filter to a specific NPI.

        is_rendering : typing.Optional[bool]
            Filter to only rendering providers.

        is_billing : typing.Optional[bool]
            Filter to only billing providers.

        page_token : typing.Optional[PageToken]
            The page token to continue paging through a previous request.

        sort : typing.Optional[OrganizationProviderSortOptions]
            Defaults to PROVIDER_NAME_ASC.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderPageV2

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient
        from candid.resources.organization_providers.resources.v_2 import (
            OrganizationProviderSortOptions,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.organization_providers.v_3.get_multi(
                limit=100,
                search_term="john",
                npi="1234567890",
                is_rendering=True,
                is_billing=True,
                page_token="eyJ0b2tlbiI6IjEiLCJwYWdlX3Rva2VuIjoiMiJ9",
                sort=OrganizationProviderSortOptions.PROVIDER_NAME_ASC,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_multi(
            limit=limit,
            search_term=search_term,
            npi=npi,
            is_rendering=is_rendering,
            is_billing=is_billing,
            page_token=page_token,
            sort=sort,
            request_options=request_options,
        )
        return _response.data

    async def create(
        self, *, request: OrganizationProviderCreateV2, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationProviderV2:
        """
        Parameters
        ----------
        request : OrganizationProviderCreateV2

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderV2

        Examples
        --------
        import asyncio

        from candid import AsyncCandidApiClient
        from candid.resources.commons import State
        from candid.resources.identifiers import (
            IdentifierCode,
            IdentifierCreate,
            IdentifierValue_MedicareProviderIdentifier,
        )
        from candid.resources.organization_providers.resources.v_2 import (
            LicenseType,
            ProviderType,
        )
        from candid.resources.organization_providers.resources.v_3 import (
            OrganizationProviderCreateV2,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.organization_providers.v_3.create(
                request=OrganizationProviderCreateV2(
                    npi="npi",
                    is_rendering=True,
                    is_billing=True,
                    provider_type=ProviderType.INDIVIDUAL,
                    license_type=LicenseType.MD,
                    qualifications=[
                        IdentifierCreate(
                            identifier_code=IdentifierCode.MCR,
                            identifier_value=IdentifierValue_MedicareProviderIdentifier(
                                state=State.AA,
                                provider_number="provider_number",
                            ),
                        ),
                        IdentifierCreate(
                            identifier_code=IdentifierCode.MCR,
                            identifier_value=IdentifierValue_MedicareProviderIdentifier(
                                state=State.AA,
                                provider_number="provider_number",
                            ),
                        ),
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(request=request, request_options=request_options)
        return _response.data

    async def update(
        self,
        organization_provider_id: OrganizationProviderId,
        *,
        request: OrganizationProviderUpdateV2,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationProviderV2:
        """
        Parameters
        ----------
        organization_provider_id : OrganizationProviderId

        request : OrganizationProviderUpdateV2

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationProviderV2

        Examples
        --------
        import asyncio
        import uuid

        from candid import AsyncCandidApiClient
        from candid.resources.organization_providers.resources.v_3 import (
            OrganizationProviderUpdateV2,
        )

        client = AsyncCandidApiClient(
            client_id="YOUR_CLIENT_ID",
            client_secret="YOUR_CLIENT_SECRET",
        )


        async def main() -> None:
            await client.organization_providers.v_3.update(
                organization_provider_id=uuid.UUID(
                    "d5e9c84f-c2b2-4bf4-b4b0-7ffd7a9ffc32",
                ),
                request=OrganizationProviderUpdateV2(),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            organization_provider_id, request=request, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .mutable_tag import MutableTag
from .tag import Tag
from .tag_page import TagPage

__all__ = ["MutableTag", "Tag", "TagPage"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ........core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....common.types.address import Address
from .....common.types.contact_point import ContactPoint
from .....common.types.human_name import HumanName


class Guarantor(UniversalBaseModel):
    name: HumanName
    telecom: typing.Optional[ContactPoint] = None
    email: typing.Optional[str] = None
    birth_date: dt.date
    address: Address

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .expected_network_status_check_error import ExpectedNetworkStatusCheckError

__all__ = ["ExpectedNetworkStatusCheckError"]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .credentialing_span_status import CredentialingSpanStatus
from .provider_credentialing_span import ProviderCredentialingSpan
from .provider_credentialing_span_id import ProviderCredentialingSpanId
from .provider_credentialing_span_page import ProviderCredentialingSpanPage

__all__ = [
    "CredentialingSpanStatus",
    "ProviderCredentialingSpan",
    "ProviderCredentialingSpanId",
    "ProviderCredentialingSpanPage",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import AuthGetTokenResponse, TooManyRequestsErrorType
from .errors import TooManyRequestsError

__all__ = ["AuthGetTokenResponse", "TooManyRequestsError", "TooManyRequestsErrorType"]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    CredentialingSpanStatus,
    ProviderCredentialingSpan,
    ProviderCredentialingSpanId,
    ProviderCredentialingSpanPage,
)

__all__ = [
    "CredentialingSpanStatus",
    "ProviderCredentialingSpan",
    "ProviderCredentialingSpanId",
    "ProviderCredentialingSpanPage",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    Task,
    TaskAction,
    TaskActionExecutionMethod,
    TaskActionExecutionMethod_CloseTask,
    TaskActionType,
    TaskActions,
    TaskAssignment,
    TaskCreateV3,
    TaskNote,
    TaskPage,
    TaskSortOptions,
    TaskUpdateV3,
    TaskUpdatedToDeprecatedStatusErrorType,
)
from .errors import TaskUpdatedToDeprecatedStatusError

__all__ = [
    "Task",
    "TaskAction",
    "TaskActionExecutionMethod",
    "TaskActionExecutionMethod_CloseTask",
    "TaskActionType",
    "TaskActions",
    "TaskAssignment",
    "TaskCreateV3",
    "TaskNote",
    "TaskPage",
    "TaskSortOptions",
    "TaskUpdateV3",
    "TaskUpdatedToDeprecatedStatusError",
    "TaskUpdatedToDeprecatedStatusErrorType",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .......core.api_error import ApiError
from .......core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .......core.http_response import AsyncHttpResponse, HttpResponse
from .......core.jsonable_encoder import jsonable_encoder
from .......core.pydantic_utilities import parse_obj_as
from .......core.request_options import RequestOptions
from ....common.types.note_id import NoteId
from .types.mutable_note import MutableNote
from .types.note import Note

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawV1Client:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def get(self, id: NoteId, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[Note]:
        """
        Gets a note by NoteId.

        Parameters
        ----------
        id : NoteId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Note]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"notes/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Note,
                parse_obj_as(
                    type_=Note,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self, *, request: MutableNote, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Note]:
        """
        Adds a new note.

        Parameters
        ----------
        request : MutableNote

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Note]
        """
        _response = self._client_wrapper.httpx_client.request(
            "notes/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Note,
                parse_obj_as(
                    type_=Note,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self, id: NoteId, version: str, *, request: MutableNote, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Note]:
        """
        Updates a note. The path must contain the most recent version to prevent races.

        Parameters
        ----------
        id : NoteId

        version : str

        request : MutableNote

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Note]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"notes/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Note,
                parse_obj_as(
                    type_=Note,  # type: ignore
                    object_=_response_json,
                ),
            )
            return HttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def deactivate(
        self, id: NoteId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Sets a note as deactivated.  The path must contain the most recent version to prevent races.

        Parameters
        ----------
        id : NoteId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"notes/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return HttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawV1Client:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def get(
        self, id: NoteId, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Note]:
        """
        Gets a note by NoteId.

        Parameters
        ----------
        id : NoteId

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Note]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"notes/v1/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Note,
                parse_obj_as(
                    type_=Note,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self, *, request: MutableNote, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Note]:
        """
        Adds a new note.

        Parameters
        ----------
        request : MutableNote

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Note]
        """
        _response = await self._client_wrapper.httpx_client.request(
            "notes/v1",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Note,
                parse_obj_as(
                    type_=Note,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self, id: NoteId, version: str, *, request: MutableNote, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Note]:
        """
        Updates a note. The path must contain the most recent version to prevent races.

        Parameters
        ----------
        id : NoteId

        version : str

        request : MutableNote

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Note]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"notes/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="PUT",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        if 200 <= _response.status_code < 300:
            _data = typing.cast(
                Note,
                parse_obj_as(
                    type_=Note,  # type: ignore
                    object_=_response_json,
                ),
            )
            return AsyncHttpResponse(response=_response, data=_data)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def deactivate(
        self, id: NoteId, version: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Sets a note as deactivated.  The path must contain the most recent version to prevent races.

        Parameters
        ----------
        id : NoteId

        version : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"notes/v1/{jsonable_encoder(id)}/{jsonable_encoder(version)}",
            base_url=self._client_wrapper.get_environment().pre_encounter,
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return AsyncHttpResponse(response=_response, data=None)
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    Diagnosis,
    DiagnosisCreate,
    DiagnosisId,
    DiagnosisNotFoundError,
    DiagnosisTypeCode,
    StandaloneDiagnosisCreate,
)
from .errors import DiagnosisNotFoundHttpError

__all__ = [
    "Diagnosis",
    "DiagnosisCreate",
    "DiagnosisId",
    "DiagnosisNotFoundError",
    "DiagnosisNotFoundHttpError",
    "DiagnosisTypeCode",
    "StandaloneDiagnosisCreate",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....invoices.resources.v_2.types.invoice_status import InvoiceStatus
from .invoice_item_info_update import InvoiceItemInfoUpdate


class ImportInvoiceUpdateRequest(UniversalBaseModel):
    customer_invoice_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    Link to the patient view of the invoice in the third-party service
    """

    status: typing.Optional[InvoiceStatus] = None
    note: typing.Optional[str] = None
    due_date: typing.Optional[dt.date] = None
    items: typing.Optional[InvoiceItemInfoUpdate] = pydantic.Field(default=None)
    """
    None here represents there is no update to the invoice items
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class BenefitType(str, enum.Enum):
    DEDUCTIBLE = "DEDUCTIBLE"
    DEDUCTIBLE_CONTRACT = "DEDUCTIBLE_CONTRACT"
    DEDUCTIBLE_REMAINING = "DEDUCTIBLE_REMAINING"
    DEDUCTIBLE_YEAR_TO_DATE = "DEDUCTIBLE_YEAR_TO_DATE"
    OOP_MAX = "OOP_MAX"
    OOP_MAX_CONTRACT = "OOP_MAX_CONTRACT"
    OOP_MAX_REMAINING = "OOP_MAX_REMAINING"
    OOP_MAX_YEAR_TO_DATE = "OOP_MAX_YEAR_TO_DATE"
    COPAY = "COPAY"
    COINSURANCE = "COINSURANCE"
    _UNKNOWN = "__BENEFITTYPE_UNKNOWN__"
    """
    This member is used for forward compatibility. If the value is not recognized by the enum, it will be stored here, and the raw value is accessible through `.value`.
    """

    @classmethod
    def _missing_(cls, value: typing.Any) -> "BenefitType":
        unknown = cls._UNKNOWN
        unknown._value_ = value
        return unknown

    def visit(
        self,
        deductible: typing.Callable[[], T_Result],
        deductible_contract: typing.Callable[[], T_Result],
        deductible_remaining: typing.Callable[[], T_Result],
        deductible_year_to_date: typing.Callable[[], T_Result],
        oop_max: typing.Callable[[], T_Result],
        oop_max_contract: typing.Callable[[], T_Result],
        oop_max_remaining: typing.Callable[[], T_Result],
        oop_max_year_to_date: typing.Callable[[], T_Result],
        copay: typing.Callable[[], T_Result],
        coinsurance: typing.Callable[[], T_Result],
        _unknown_member: typing.Callable[[str], T_Result],
    ) -> T_Result:
        if self is BenefitType.DEDUCTIBLE:
            return deductible()
        if self is BenefitType.DEDUCTIBLE_CONTRACT:
            return deductible_contract()
        if self is BenefitType.DEDUCTIBLE_REMAINING:
            return deductible_remaining()
        if self is BenefitType.DEDUCTIBLE_YEAR_TO_DATE:
            return deductible_year_to_date()
        if self is BenefitType.OOP_MAX:
            return oop_max()
        if self is BenefitType.OOP_MAX_CONTRACT:
            return oop_max_contract()
        if self is BenefitType.OOP_MAX_REMAINING:
            return oop_max_remaining()
        if self is BenefitType.OOP_MAX_YEAR_TO_DATE:
            return oop_max_year_to_date()
        if self is BenefitType.COPAY:
            return copay()
        if self is BenefitType.COINSURANCE:
            return coinsurance()
        return _unknown_member(self._value_)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ......core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....commons.types.schema_id import SchemaId


class SchemaInstance(UniversalBaseModel):
    """
    Examples
    --------
    import uuid

    from candid.resources.custom_schemas.resources.v_1 import SchemaInstance

    SchemaInstance(
        schema_id=uuid.UUID(
            "ec096b13-f80a-471d-aaeb-54b021c9d582",
        ),
        content={
            "provider_category": "internist",
            "is_urgent_care": True,
            "bmi": 24.2,
            "age": 38,
        },
    )
    """

    schema_id: SchemaId = pydantic.Field()
    """
    The schema to which the content must adhere.
    """

    content: typing.Dict[str, typing.Optional[typing.Any]] = pydantic.Field()
    """
    A set of key-value pairs that adhere to the naming and type convention of the schema. Not all keys in the schema must be included, but attaching any key that does not exist in the schema or attaching a key with the incorrect value type will result in errors.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ....core.pydantic_utilities import IS_PYDANTIC_V2
from ...commons.types.email import Email
from ...commons.types.phone_number import PhoneNumber
from ...non_insurance_payers.resources.v_1.types.non_insurance_payer import NonInsurancePayer
from .individual_id import IndividualId
from .patient_base import PatientBase
from .patient_non_insurance_payer_info import PatientNonInsurancePayerInfo


class Patient(PatientBase):
    """
    Examples
    --------
    import datetime
    import uuid

    from candid.resources.commons import (
        PhoneNumber,
        PhoneNumberType,
        State,
        StreetAddressShortZip,
    )
    from candid.resources.individual import (
        Gender,
        Patient,
        PatientNonInsurancePayerInfo,
    )
    from candid.resources.non_insurance_payers.resources.v_1 import (
        NonInsurancePayer,
    )

    Patient(
        individual_id=uuid.UUID(
            "93ddbebf-4956-4482-9a6c-21499b7e4e5d",
        ),
        phone_numbers=[
            PhoneNumber(
                number="1234567890",
                type=PhoneNumberType.HOME,
            )
        ],
        non_insurance_payers=[
            NonInsurancePayer(
                non_insurance_payer_id=uuid.UUID(
                    "eb7623ab-d5bc-4b25-b257-2b8fcec578de",
                ),
                name="Sunrise Foundation",
                category="Foundation",
                description="Sunrise Foundation is a non-profit organization that provides financial assistance to patients in need.",
                enabled=True,
                address=StreetAddressShortZip(
                    address_1="123 Main St",
                    city="San Francisco",
                    state=State.CA,
                    zip_code="94105",
                ),
                clinical_trials=[],
            )
        ],
        non_insurance_payers_info=[
            PatientNonInsurancePayerInfo(
                non_insurance_payer=NonInsurancePayer(
                    non_insurance_payer_id=uuid.UUID(
                        "eb7623ab-d5bc-4b25-b257-2b8fcec578de",
                    ),
                    name="Sunrise Foundation",
                    category="Foundation",
                    description="Sunrise Foundation is a non-profit organization that provides financial assistance to patients in need.",
                    enabled=True,
                    address=StreetAddressShortZip(
                        address_1="123 Main St",
                        city="San Francisco",
                        state=State.CA,
                        zip_code="94105",
                    ),
                    clinical_trials=[],
                ),
                member_id="123456789",
                clinical_trial_info=[],
            )
        ],
        phone_consent=True,
        email="johndoe@joincandidhealth.com",
        email_consent=True,
        external_id="49460F77-6456-41F1-AC6D-0AED08614D39",
        date_of_birth=datetime.date.fromisoformat(
            "2000-01-01",
        ),
        address=StreetAddressShortZip(
            address_1="123 Main St",
            address_2="Apt 1",
            city="New York",
            state=State.NY,
            zip_code="10001",
            zip_plus_four_code="1234",
        ),
        first_name="John",
        last_name="Doe",
        gender=Gender.MALE,
    )
    """

    individual_id: IndividualId
    phone_numbers: typing.List[PhoneNumber]
    phone_consent: bool
    email: typing.Optional[Email] = None
    email_consent: bool
    non_insurance_payers: typing.List[NonInsurancePayer]
    non_insurance_payers_info: typing.List[PatientNonInsurancePayerInfo]

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .raw_client import AsyncRawSuperbillsClient, RawSuperbillsClient
from .resources.v_1.client import AsyncV1Client, V1Client


class SuperbillsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSuperbillsClient(client_wrapper=client_wrapper)
        self.v_1 = V1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSuperbillsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSuperbillsClient
        """
        return self._raw_client


class AsyncSuperbillsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSuperbillsClient(client_wrapper=client_wrapper)
        self.v_1 = AsyncV1Client(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSuperbillsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSuperbillsClient
        """
        return self._raw_client
